/***************************************************************************
 *   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation version 2.1.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
 ***************************************************************************/
#ifndef BITTYPE_ERROR_H
#define BITTYPE_ERROR_H

#include <sstream>

#include <bit/error.h>

namespace bit
  {

  /**
   * Base class for all type errors.
   * @author Rick L Vinyard Jr
   * @ingroup errors
   */
  class type_error: public bit_error
    {
    public:
      type_error(): bit_error("bit:Type: Unknown type error.")
      { }
      type_error(const std::string s): bit_error(s)
      { }
    }
  ;

  namespace error
    {


    namespace type
      {

      /**
       * Integer length: Integer length must be <= 64
       * @author Rick L Vinyard Jr
       * @ingroup errors
       */
      class integer_length: public type_error
        {
        public:
          integer_length(): type_error("bit:Type: Integer length must be <= 64.")
          { }
        }
      ;

      /**
       * Bad floating point length: length must be 32 or 64
       * @author Rick L Vinyard Jr
       * @ingroup errors
       */
      class floating_point_length: public type_error
      {
        public:
          floating_point_length(): type_error("bit:Type: Bad floating point length. Length must be 32 or 64.")
          { }
      }
      ;

    }
  }
}
#endif
