/***************************************************************************
*   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
*   rvinyard@cs.nmsu.edu                                                  *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU Lesser General Public License as        *
*   published by the Free Software Foundation version 2.1.                *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU Lesser General Public      *
*   License along with this library; if not, write to the                 *
*   Free Software Foundation, Inc.,                                       *
*   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
***************************************************************************/
#ifndef BITFIELDTYPE_H
#define BITFIELDTYPE_H

namespace bit {

  typedef enum Type { TYPE_NONE, TYPE_INTEGER, TYPE_FLOATING, TYPE_ASCII, TYPE_UTF8, TYPE_BCD } Type;

  typedef enum ByteOrder { BYTEORDER_NETWORK, BYTEORDER_BIG_ENDIAN, BYTEORDER_LITTLE_ENDIAN } ByteOrder;

  typedef enum Sign { SIGN_UNSIGNED, SIGN_TWOS_COMPLEMENT, SIGN_ONES_COMPLEMENT, SIGN_BIT } Sign;

/**
	@author Rick L. Vinyard, Jr. <rvinyard@cs.nmsu.edu>
*/
class FieldType{
public:

  FieldType(Type type=TYPE_NONE, ByteOrder byte_order=BYTEORDER_NETWORK, Sign sign=SIGN_UNSIGNED);

    ~FieldType();

    Type type() const;
    void set_type(Type);
    const char* type_string() const;

    ByteOrder byte_order() const;
    void set_byte_order(ByteOrder);
    const char* byte_order_string() const;

    Sign sign() const;
    void set_sign(Sign);
    const char* sign_string() const;

    static const char* type_string(Type);
    static const char* byte_order_string(ByteOrder);
    static const char* sign_string(Sign);
    
  protected:
    Type m_type;
    ByteOrder m_byte_order;
    Sign m_sign;

    static const char* m_type_string[];
    static const char* m_byte_order_string[];
    static const char* m_sign_string[];
};

}

#endif
