/***************************************************************************
*   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
*   rvinyard@cs.nmsu.edu                                                  *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU Lesser General Public License as        *
*   published by the Free Software Foundation version 2.1.                *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU Lesser General Public      *
*   License along with this library; if not, write to the                 *
*   Free Software Foundation, Inc.,                                       *
*   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
***************************************************************************/
#ifndef BITRECORDBASE_H
#define BITRECORDBASE_H

#include <bit/fieldbase.h>

namespace bit {

/**
 * Provides a common interface to both Record and RecordVector classes
 * 
 * @ingroup records
 * @author Rick L. Vinyard, Jr. <rvinyard@cs.nmsu.edu>
 */
class RecordBase : public FieldBase
{
public:
  typedef boost::shared_ptr<RecordBase> pointer;
  
  RecordBase( std::string name="",
              size_t length_units=BITS,
              int offset=0,
              size_t offset_units=BITS,
              std::string descr=""
            );

    virtual ~RecordBase();

    virtual size_t length_units() const;

    virtual void set_length_units(size_t units);

    /** @return the offset of this object in the offset units of this object */
    virtual int offset() const;

    virtual int offset(size_t units) const;

    virtual void set_offset(int offset);

    virtual size_t offset_units() const;

    virtual void set_offset_units(size_t units);

      /**
     * Returns the string name associated with this object
     *
     * This pure virtual method is reimplemented by children to
     * provide the proper notation, such as the array notation of
     * elements, by children.
       */
    virtual std::string name() const;

    virtual std::string name(int depth) const;

    /** Sets the name of this record */
    virtual void set_name(const std::string& name);

      /**
     * Returns the string description of this object.
       */
    virtual std::string description() const;

    /** Sets the description of this record */
    virtual void set_description(const std::string& description);

    virtual void add_field(FieldBase::pointer child) = 0;

    virtual void remove_field(const std::string& name) = 0;

  protected:
    std::string m_name;
    size_t m_length_units;
    int m_offset;
    size_t m_offset_units;
    std::string m_description;

};

}

#endif
